
#ifndef __FIELDOFFRACTIONS_H
    #define __FIELDOFFRACTIONS_H

    #define Fraction FieldOfFractions
    #define frac FieldOfFractions

    #include<sstream>

    template<class IntegralDomain> class FieldOfFractions
    {
        protected:
            IntegralDomain Numerator; // Zaehler
            IntegralDomain Denominator; // Nenner
        public:
            FieldOfFractions();
            FieldOfFractions(const FieldOfFractions<IntegralDomain>&); // copy constuctor
            FieldOfFractions(const IntegralDomain& Numerator, const IntegralDomain& Denominator);
            FieldOfFractions(const IntegralDomain& Numerator);
            void Reduce();           // THIS ONLY WORKS FOR EUCLIDIAN DOMAINS!!!
            IntegralDomain getNumerator() const;
            IntegralDomain getDenominator() const;
            bool setNumerator(const IntegralDomain&);
            bool setDenominator(const IntegralDomain&);
            //========================================================================== Operators
            FieldOfFractions<IntegralDomain> operator + (const FieldOfFractions<IntegralDomain>& x) const;
            FieldOfFractions<IntegralDomain> operator - (const FieldOfFractions<IntegralDomain>& x) const;
            FieldOfFractions<IntegralDomain> operator / (const FieldOfFractions<IntegralDomain>& x) const;
            FieldOfFractions<IntegralDomain> operator * (const FieldOfFractions<IntegralDomain>& x) const;
            //=============================================================== Assignment Operators
            bool operator += (const FieldOfFractions<IntegralDomain>& x);
            bool operator -= (const FieldOfFractions<IntegralDomain>& x);
            bool operator /= (const FieldOfFractions<IntegralDomain>& x);
            bool operator *= (const FieldOfFractions<IntegralDomain>& x);
            FieldOfFractions<IntegralDomain> operator ++ (int); // Postfix
            FieldOfFractions<IntegralDomain> operator -- (int);
            FieldOfFractions<IntegralDomain> operator ++ ();    // Prefix
            FieldOfFractions<IntegralDomain> operator -- ();
            bool operator = (const FieldOfFractions<IntegralDomain>& x);
            //=========================================================================== Relators
            bool operator == (const FieldOfFractions<IntegralDomain>& x) const;
            bool operator != (const FieldOfFractions<IntegralDomain>& x) const;
            bool operator >  (const FieldOfFractions<IntegralDomain>& x) const;
            bool operator <  (const FieldOfFractions<IntegralDomain>& x) const;
            bool operator >= (const FieldOfFractions<IntegralDomain>& x) const;
            bool operator <= (const FieldOfFractions<IntegralDomain>& x) const;
            std::string LaTeX() const;
    };



    template<class IntegralDomain> FieldOfFractions<IntegralDomain>::FieldOfFractions()
    {
        setNumerator(IntegralDomain(0));
        setDenominator(IntegralDomain(1));
    }
    
    template<class IntegralDomain> FieldOfFractions<IntegralDomain>::FieldOfFractions(const FieldOfFractions<IntegralDomain>& tocopy)
    {
        setNumerator(tocopy.Numerator);
        setDenominator(tocopy.Denominator);
    }
    
    template<class IntegralDomain> FieldOfFractions<IntegralDomain>::FieldOfFractions(const IntegralDomain& Numerator, const IntegralDomain& Denominator)
    {
        setNumerator(Numerator);
        setDenominator(IntegralDomain(1));
        setDenominator(Denominator);
    }
    
    template<class IntegralDomain> FieldOfFractions<IntegralDomain>::FieldOfFractions(const IntegralDomain& Numerator)
    {
        setNumerator(Numerator);
        setDenominator(IntegralDomain(1));
    }
    
    template<class IntegralDomain> void FieldOfFractions<IntegralDomain>::Reduce()
    {
        IntegralDomain zero = IntegralDomain(0);
        IntegralDomain d = Denominator;
        IntegralDomain n = Numerator;
        IntegralDomain nmodd;             // Euclidian algorithm (determine greatest common divisor)
        while(d != zero)                  // THIS ONLY WORKS FOR EUCLIDIAN DOMAINS!!!!
        {
            nmodd = n % d;
            n = d;
            d = nmodd;
        }
        Denominator = Denominator / n;    // now n is the greatest common divisor
        Numerator = Numerator / n;
    }
    
    template<class IntegralDomain> bool FieldOfFractions<IntegralDomain>::setNumerator(const IntegralDomain& X)
    {
        return Numerator=X;
    }
    
    template<class IntegralDomain> bool FieldOfFractions<IntegralDomain>::setDenominator(const IntegralDomain& X)
    {
        if(X == IntegralDomain(0))
            return(false);
        return(Denominator=X);
    }
    
    template<class IntegralDomain> IntegralDomain FieldOfFractions<IntegralDomain>::getNumerator() const
    {
        return(Numerator);
    }
    
    template<class IntegralDomain> IntegralDomain FieldOfFractions<IntegralDomain>::getDenominator() const
    {
        return(Denominator);
    }
    
    //========================================================================== Operators

    template<class IntegralDomain> FieldOfFractions<IntegralDomain> FieldOfFractions<IntegralDomain>::operator + (const FieldOfFractions<IntegralDomain>& x) const
    {
        return(FieldOfFractions<IntegralDomain>(Numerator*x.Denominator+x.Numerator*Denominator, Denominator*x.Denominator));
    }
    
    template<class IntegralDomain> FieldOfFractions<IntegralDomain> FieldOfFractions<IntegralDomain>::operator - (const FieldOfFractions<IntegralDomain>& x) const
    {
        return(FieldOfFractions<IntegralDomain>(Numerator*x.Denominator-x.Numerator*Denominator, Denominator*x.Denominator));
    }
    
    template<class IntegralDomain> FieldOfFractions<IntegralDomain> FieldOfFractions<IntegralDomain>::operator / (const FieldOfFractions<IntegralDomain>& x) const
    {
        return(FieldOfFractions<IntegralDomain>(Numerator*x.Denominator, Denominator*x.Numerator));
    }
    
    template<class IntegralDomain> FieldOfFractions<IntegralDomain> FieldOfFractions<IntegralDomain>::operator * (const FieldOfFractions<IntegralDomain>& x) const
    {
        return(FieldOfFractions<IntegralDomain>(Numerator*x.Numerator, Denominator*x.Denominator));
    }

    //=============================================================== Assignment Operators

    template<class IntegralDomain> bool FieldOfFractions<IntegralDomain>::operator = (const FieldOfFractions<IntegralDomain>& x)
    {
        return( Numerator = x.Numerator  &&  Denominator = x.Denominator );
    }
    
    template<class IntegralDomain> bool FieldOfFractions<IntegralDomain>::operator += (const FieldOfFractions<IntegralDomain>& x)
    {
        return operator=(operator+(x));
    }
    
    template<class IntegralDomain> bool FieldOfFractions<IntegralDomain>::operator -= (const FieldOfFractions<IntegralDomain>& x)
    {
        return operator=(operator-(x));
    }
    
    template<class IntegralDomain> bool FieldOfFractions<IntegralDomain>::operator /= (const FieldOfFractions<IntegralDomain>& x)
    {
        return operator=(operator/(x));
    }
    
    template<class IntegralDomain> bool FieldOfFractions<IntegralDomain>::operator *= (const FieldOfFractions<IntegralDomain>& x)
    {
        return operator=(operator*(x));
    }
    
    template<class IntegralDomain> FieldOfFractions<IntegralDomain> FieldOfFractions<IntegralDomain>::operator ++ (int)
    {
        Numerator = Numerator + Denominator;
        return(FieldOfFractions<IntegralDomain>(Numerator - Denominator, Denominator));
    }
    
    template<class IntegralDomain> FieldOfFractions<IntegralDomain> FieldOfFractions<IntegralDomain>::operator -- (int)
    {
        Numerator = Numerator - Denominator;
        return(FieldOfFractions<IntegralDomain>(Numerator + Denominator, Denominator));
    }
    
    template<class IntegralDomain> FieldOfFractions<IntegralDomain> FieldOfFractions<IntegralDomain>::operator ++ ()
    {
        Numerator = Numerator + Denominator;
        return(*this);
    }
    
    template<class IntegralDomain> FieldOfFractions<IntegralDomain> FieldOfFractions<IntegralDomain>::operator -- ()
    {
        Numerator = Numerator - Denominator;
        return(*this);
    }

    //=========================================================================== Relators

    template<class IntegralDomain> bool FieldOfFractions<IntegralDomain>::operator ==(const FieldOfFractions<IntegralDomain>& x) const
    {
        return( Numerator*x.Denominator == x.Numerator*Denominator );
    }
    
    template<class IntegralDomain> bool FieldOfFractions<IntegralDomain>::operator !=(const FieldOfFractions<IntegralDomain>& x) const
    {
        return( Numerator*x.Denominator != x.Numerator*Denominator );
    }
    
    template<class IntegralDomain> bool FieldOfFractions<IntegralDomain>::operator > (const FieldOfFractions<IntegralDomain>& x) const
    {
        return( Numerator*x.Denominator > x.Numerator*Denominator );
    }
    
    template<class IntegralDomain> bool FieldOfFractions<IntegralDomain>::operator < (const FieldOfFractions<IntegralDomain>& x) const
    {
        return( Numerator*x.Denominator < x.Numerator*Denominator );
    }
    
    template<class IntegralDomain> bool FieldOfFractions<IntegralDomain>::operator >= (const FieldOfFractions<IntegralDomain>& x) const
    {
        return( Numerator*x.Denominator >= x.Numerator*Denominator );
    }
    
    template<class IntegralDomain> bool FieldOfFractions<IntegralDomain>::operator <= (const FieldOfFractions<IntegralDomain>& x) const
    {
        return( Numerator*x.Denominator <= x.Numerator*Denominator );
    }

    //============================================================================ Streams
      
    template<class IntegralDomain> istream& operator >> (istream & is, FieldOfFractions<IntegralDomain>& F)
    {
        IntegralDomain n,d;
        is >> n >> d;
        F.setNumerator(n);
        F.setDenominator(d);
        return is;
    }
    
    template<class IntegralDomain> ostream& operator << (ostream & os, const FieldOfFractions<IntegralDomain>& F)
    {
        return os << F.getNumerator() << '\n' << F.getDenominator() << '\n';
    }
    
    template<class IntegralDomain> std::string FieldOfFractions<IntegralDomain>::LaTeX() const
    {
        std::stringstream buffer(stringstream::out);
        buffer << "\\frac{ " << Numerator << " }{ " << Denominator << " } ";
        return buffer.str();
    }

#endif
